"""
nomad.models.node

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/nodes.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from nomad.models.resource import NetworkResource, NodeDeviceResource, Resources
from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Address:
    """ Address model """

    family: str
    alias: str
    address: str
    reserved_ports: str
    gateway: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeNetwork:
    """ NodeNetwork model """

    mode: str
    device: str
    mac_address: str
    speed: int
    addresses: Optional[list[Address]]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeReservedNetworkResources:
    """ NodeReservedNetworkResources model """

    reserved_host_ports: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeEvent:
    """ NodeEvent model """

    message: str
    subsystem: str
    details: Optional[dict[str, str]]
    timestamp: str
    create_index: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class CSITopology:
    """ CSITopology model """

    segments: dict[str, str]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class CSINodeInfo:
    """ CSINodeInfo model """

    id: str = field(metadata={"serde_rename": "ID"})
    max_volumes: int
    accessible_topology: CSITopology
    requires_node_stage_volume: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class CSIControllerInfo:
    """ CSIControllerInfo model """

    supports_read_only_attach: bool
    supports_attach_detach: bool
    supports_list_volumes: bool
    supports_list_volumes_attached_nodes: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class CSIInfo:
    """ CSIInfo model """

    plugin_id: str = field(metadata={"serde_rename": "PluginID"})
    alloc_id: str = field(metadata={"serde_rename": "AllocID"})
    healthy: bool
    health_description: str
    update_time: str
    requires_controller_plugin: bool
    requires_topologies: bool
    controller_info: Optional[CSIControllerInfo]
    node_info: Optional[CSINodeInfo]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class DrainStrategy:
    """ DrainStrategy model """

    force_deadline: str
    started_at: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class DrainSpec:
    """ DrainSpec model """

    deadline: int
    ignore_system_jobs: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class DriverInfo:
    """ DriverInfo model """

    attributes: Optional[dict[str, str]]
    detected: bool
    healthy: bool
    health_description: str
    update_time: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class HostVolumeInfo:
    """ HostVolumeInfo model """

    path: str
    read_only: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeCpuResources:
    """ NodeCpuResources model """

    cpu_shares: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeMemoryResources:
    """ NodeMemoryResources model """

    memory_mb: int = field(metadata={"serde_rename": "MemoryMB"})


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeDiskResources:
    """ NodeDiskResources model """

    disk_mb: int = field(metadata={"serde_rename": "DiskMB"})


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeReservedResources:
    """ NodeReservedResources model """

    cpu: NodeCpuResources
    memory: NodeMemoryResources
    disk: NodeDiskResources
    # networks: NodeReservedResources


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeResources:
    """ NodeResources model """

    cpu: NodeCpuResources
    memory: "NodeMemoryResources"
    disk: "NodeDiskResources"
    networks: Optional[list[NetworkResource]]
    devices: Optional[list[NodeDeviceResource]]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Node:
    """ Node model """

    id: str = field(metadata={"serde_rename": "ID"})
    datacenter: str
    name: str
    http_addr: str = field(metadata={"serde_rename": "HTTPAddr"})
    tls_enabled: bool = field(metadata={"serde_rename": "TLSEnabled"})
    attributes: dict[str, str]
    resources: Resources
    reserved: Resources
    node_resources: NodeResources
    reserved_resources: NodeReservedResources
    links: dict[str, str]
    meta: dict[str, str]
    node_class: str
    drain: bool
    drain_strategy: Optional[DrainStrategy]
    scheduling_eligibility: str
    status: str
    status_description: str
    status_updated_at: int
    events: list[NodeEvent]
    drivers: Optional[dict[str, DriverInfo]]
    host_volumes: Optional[dict[str, HostVolumeInfo]]
    # csi_controller_plugins: Optional[dict[str, CSIInfo]]
    # csi_node_plugins: Optional[dict[str, CSIInfo]]
    create_index: int
    modify_index: int



import json
import logging
from dataclasses import dataclass, field
from typing import AsyncGenerator, Optional

import aiohttp
from nomad.models import WriteOptions, event_stream
from nomad.models.job import Job, RegisterOptions
from serde.json import from_json

log = logging.getLogger("nomad")


@dataclass
class Config:
    """ Nomad Client Config """

    timeout: int = 60 * 60 * 24 * 365 * 10  # Timeout for HTTP client.
    token: str = ""  # Nomad API token.


@dataclass
class Client:
    """
    Nomad API Async Client
    """

    addr: str  # Nomad Address.
    cfg: Optional[Config] = field(default_factory=Config)
    ses: Optional[aiohttp.ClientSession] = field(init=False, default=None)

    async def __aenter__(self):
        opts = {"timeout": aiohttp.ClientTimeout(total=self.cfg.timeout)}

        headers = {}
        if self.cfg.token:
            headers["X-Nomad-Token"] = self.cfg.token

        self.ses = aiohttp.ClientSession(headers=headers, **opts)
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.ses.close()

    def _check_session(self) -> aiohttp.ClientSession:
        """
        Check session is established
        """
        if not self.ses:
            raise Exception("Client session isn't started")
        return self.ses

    async def event_stream(self) -> AsyncGenerator[event_stream.Events, None]:
        """
        Subscribe event_stream
        """
        ses = self._check_session()
        async with ses.get(self.addr + "/v1/event/stream") as reader:
            async for raw in reader.content:
                line = raw.strip().decode()
                log.debug(f"Raw response: {line}")

                if not line or not json.loads(line):
                    continue
                yield from_json(event_stream.Events, line)

    async def register_job(
        self, job: Job, opts: Optional[RegisterOptions] = None, wopts: Optional[WriteOptions] = None
    ):
        pass

    async def job(self, name: str) -> Job:
        """
        Get job
        """
        ses = self._check_session()
        async with ses.get(self.addr + f"/v1/job/{name}") as res:
            text = await res.text()
            status = res.status
            if status != 200:
                raise Exception(text)
            return from_json(Job, text)

    async def jobs(self) -> list[Job]:
        """
        Get jobs
        """
        ses = self._check_session()
        async with ses.get(self.addr + "/v1/jobs") as res:
            text = await res.text()
            status = res.status
            if status != 200:
                raise Exception(text)
            return from_json(list[Job], text)



"""
nomad.models.service

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/services.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Service:
    id: str = field(metadata={"serde_rename": "ID"})
    name: str
    tags: Optional[list[str]]
    canary_tags: Optional[list[str]]
    enable_tag_override: bool
    port_label: str
    address_mode: str
    # checks: Optional[list[]]
    # check_restart: Optional[list[]]
    # connect: Optional[list[]]
    meta: Optional[dict[str, str]]
    canary_meta: Optional[dict[str, str]]
    task_name: str



"""
nomad.models.constraint

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/constraint.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass

from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Constraint:
    """ Constraint model """

    l_target: str
    r_target: str
    operand: str



"""
nomad.models.event_stream

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/event_stream.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass
from typing import Optional, Union

from nomad.models.allocation import Allocation
from nomad.models.deployment import Deployment
from nomad.models.evaluation import Evaluation
from nomad.models.job import Job
from nomad.models.node import Node
from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Event:
    """ Event model """

    topic: str
    type: str
    key: str
    filter_keys: Optional[list[str]]
    index: Optional[int]
    payload: Optional[dict[str, Union[Evaluation, Allocation, Deployment, Job, Node]]]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Events:
    """ Events model """

    index: int
    events: Optional[list[Event]]



"""
nomad.models.task

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/tasks.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from serde import deserialize, serialize

from .constraint import Constraint
from .resource import Resources
from .service import Service


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Affinity:
    l_target: str
    r_target: str
    operand: str
    weight: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class RestartPolicy:
    interval: int
    attempts: str
    delay: int
    mode: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class LogConfig:
    max_files: int
    max_file_size_mb: int = field(metadata={"serde_rename": "MaxFileSizeMB"})


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class TaskArtifact:
    getter_source: Optional[str]
    getter_options: Optional[dict[str, str]]
    getter_headers: Optional[dict[str, str]]
    getter_mode: Optional[str]
    relative_dest: Optional[str]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Vault:
    policies: list[str]
    namespace: Optional[str]
    env: Optional[bool]
    change_mode: Optional[str]
    change_signal: Optional[str]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Template:
    source_path: Optional[str]
    dest_path: Optional[str]
    embedded_tmpl: Optional[str]
    change_mode: Optional[str]
    change_signal: Optional[str]
    splay: Optional[int]
    perms: Optional[str]
    left_delim: Optional[str]
    right_delim: Optional[str]
    envvars: Optional[int]
    vault_grace: Optional[int]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class VolumeMount:
    volume: str
    destination: str
    read_only: bool
    propagation_mode: str


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Task:
    name: str
    driver: str
    user: str
    # lifecycle: TaskLifecycle
    # config
    constraints: Optional[list[Constraint]]
    affinities: Optional[list[Affinity]]
    env: Optional[dict[str, str]]
    services: Optional[list[Service]]
    resources: Resources
    restart_policy: RestartPolicy
    templates: Optional[list[Template]]
    # dispatch_payload: DispatchPayloadConfig
    volume_mounts: Optional[list[VolumeMount]]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class TaskGroup:
    name: str
    count: int
    constraints: Optional[list[Constraint]]
    affinities: Optional[list[Affinity]]
    tasks: Optional[list[Task]]
    meta: Optional[dict[str, str]]
    kill_timeout: Optional[str]
    log_config: Optional[LogConfig]
    artifacts: Optional[list[TaskArtifact]]
    vault: Optional[Vault]



"""
nomad.models.evaluation

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/evaluations.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Evaluation:
    """ Evaluation model """

    id: str = field(metadata={"serde_rename": "ID"})
    priority: int
    type: str
    triggered_by: str
    namespace: str
    job_id: str = field(metadata={"serde_rename": "JobID"})
    job_modify_index: int
    node_id: str = field(metadata={"serde_rename": "NodeID"})
    node_modify_index: int
    deployment_id: str = field(metadata={"serde_rename": "DeploymentID"})
    status: str
    status_description: str
    wait: int
    # wait_until: datetime
    next_eval: str
    previous_eval: str
    blocked_eval: str
    # FailedTGAllocs
    class_eligibility: Optional[dict[str, bool]]
    escaped_computed_class: bool
    quota_limit_reached: str
    annotate_plan: bool
    queue_allocations: Optional[dict[str, int]]
    snapshot_index: int
    create_index: int
    modify_index: int
    create_time: int
    modify_time: int



"""
nomad.models.job


The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/jobs.go)
in Hashicorp Nomad project.
"""
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Optional

from nomad.models.constraint import Constraint
from nomad.models.task import Affinity, TaskGroup
from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class RegisterRequest:
    """ RegisterRequest """

    job: Job
    enforce_index: bool
    modify_index: int
    policy_override: bool
    preserve_counts: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class RegisterOptions:
    """ RegisterOptions model """

    enforce_index: bool
    modify_index: int
    policy_override: bool
    preserve_counts: bool


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Children:
    """ Children model """

    pending: int
    running: int
    dead: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Summary:
    """ Summary model """

    queued: int
    complete: int
    failed: int
    running: int
    starting: int
    lost: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class JobSummary:
    """ JobSummary model """

    job_id: str = field(metadata={"serde_rename": "JobID"})
    namespace: str
    summary: dict[str, Summary]
    create_index: int
    modify_index: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Job:
    """ Job model """

    region: Optional[str]
    namespace: Optional[str]
    id: Optional[str] = field(metadata={"serde_rename": "ID"})
    name: Optional[str]
    type: Optional[str]
    priority: int
    all_at_once: Optional[bool]
    datacenters: Optional[list[str]]
    constraints: Optional[list[Constraint]]
    affinities: Optional[list[Affinity]]
    task_groups: Optional[list[TaskGroup]]
    # update
    # multiregion
    # spreads: Optional[list[Spread]]
    # periodic
    # parameterized_job
    # reschedule
    # migrate
    meta: Optional[dict[str, str]]
    consul_token: Optional[str]
    vault_token: Optional[str]
    stop: Optional[bool]
    parent_id: Optional[str] = field(metadata={"serde_rename": "ParentID"})
    dispatched: Optional[bool]
    # payload: byte
    vault_namespace: Optional[str]
    nomad_token_id: Optional[str] = field(metadata={"serde_rename": "NomadTokenID"})
    status: Optional[str]
    status_description: Optional[str]
    stable: Optional[bool]
    version: Optional[int]
    submit_time: Optional[int]
    create_index: Optional[int]
    modify_index: Optional[int]
    job_modify_index: Optional[int]



"""
nomad.models.resource

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/resources.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from nomad.models.constraint import Constraint
from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NodeDeviceResource:
    vendor: str
    type: str
    name: str
    # instances: Optional[list[NodeDevice]]
    # attributes: Optional[dict[str, Attribute]


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class NetworkResource:
    mode: str
    device: str
    cidr: str = field(metadata={"serde_rename": "CIDR"})
    ip: str = field(metadata={"serde_rename": "IP"})
    dns: Optional[str] = field(metadata={"serde_rename": "DNS"})
    reserved_ports: Optional[list[str]]
    dynamic_ports: Optional[str]
    mbits: str = field(metadata={"serde_rename": "MBits"})


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Resources:
    cpu: Optional[int] = field(metadata={"serde_rename": "CPU"})
    memory_mb: Optional[int] = field(metadata={"serde_rename": "MemoryMB"})
    disk_mb: Optional[int] = field(metadata={"serde_rename": "DiskMB"})
    networks: Optional[list[NetworkResource]]
    devcies: Optional[list["RequestedDevice"]]
    iops: Optional[int] = field(metadata={"serde_rename": "IOPS"})


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class RequestedDevice:
    name: str
    count: Optional[int]
    constraints: Optional[list[Constraint]]
    # affinities: Optional[list[Affinity]]



"""
nomad.models.allocation

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/allocations.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from nomad.models.job import Job
from nomad.models.resource import Resources
from nomad.models.service import Service
from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Allocation:
    """ Allocation model """

    id: str = field(metadata={"serde_rename": "JobID"})
    namespace: str
    eval_id: str = field(metadata={"serde_rename": "EvalID"})
    name: str
    node_id: str = field(metadata={"serde_rename": "NodeID"})
    node_name: str
    job_id: str = field(metadata={"serde_rename": "JobID"})
    job: Job
    task_group: str
    resources: Resources
    task_resources: Optional[dict[str, Resources]]
    # allocated_resources
    services: Optional[dict[str, Service]]
    # metrics
    desired_status: str
    desired_description: str
    # desired_transition
    client_status: str
    client_description: str
    # task_states
    deployment_id: str = field(metadata={"serde_rename": "DeploymentID"})
    # deployment_status
    followup_eval_id: str = field(metadata={"serde_rename": "FollowupEvalID"})
    previous_allocation: str
    next_allocation: str
    # reschedule_tracker
    preempted_allocations: Optional[list[str]]
    preempted_by_allocation: str
    create_index: int
    modify_index: int
    alloc_modify_index: int
    create_time: int
    modify_time: int



"""
nomad.models.deployment

The models defined in this module can be found
[here](https://github.com/hashicorp/nomad/blob/v1.0.4/api/deployments.go)
in Hashicorp Nomad project.
"""
from dataclasses import dataclass, field
from typing import Optional

from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class Deployment:
    """ Deployment model """

    id: str = field(metadata={"serde_rename": "ID"})
    namespace: str
    job_id: str = field(metadata={"serde_rename": "JobID"})
    job_version: int
    job_modify_index: int
    job_spec_modify_index: int
    job_create_index: int
    is_multiregion: bool
    task_groups: Optional[dict[str, "DeploymentState"]]
    status: str
    status_description: str
    create_index: int
    modify_index: int


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class DeploymentState:
    """ DeploymentState model """

    placed_canaries: Optional[list[str]]
    auto_revert: bool
    progress_deadline: int
    # required_progress_by
    promoted: bool
    desired_canaries: int
    desired_total: int
    placed_allocs: int
    healthy_allocs: int
    unhealthy_allocs: int



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nomad', 'nomad.models']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3,<4', 'pyserde']

setup_kwargs = {
    'name': 'nomad-api',
    'version': '0.0.2',
    'description': 'Hashicorp Nomad API client written in modern Python',
    'long_description': None,
    'author': 'yukinarit',
    'author_email': 'yukinarit84@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass, field
from typing import Optional

from serde import deserialize, serialize


@deserialize(rename_all="pascalcase")
@serialize(rename_all="pascalcase")
@dataclass
class WriteOptions:
    """ WriteOptions model """

    region: str
    namespace: str
    auth_token: str
    # ctx:
